﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.Core;

namespace HeuristicLab.Problems.Scheduling.Views {
  [View("Schedule View")]
  [Content(typeof(Schedule), true)]
  public partial class ScheduleView : NamedItemView {
    public ScheduleView() {
      InitializeComponent();
    }

    public new Schedule Content {
      get { return (Schedule)base.Content; }
      set { base.Content = value; }
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        ResetGanttChart();
      } else {
        FillGanttChart(Content);
      }
    }

    private void ResetGanttChart() {
      ganttChart.Reset();
    }

    private void FillGanttChart(Schedule content) {
      //Add Resources as Categories
      int resCount = 0;
      Random random = new Random ();
      foreach (Resource r in content.Resources) {
        foreach (JSSPTask t in content.Resources[resCount].Tasks) {
          string categoryName = "ScheduleTasks";
          string toolTip = "Task#" + t.TaskNr.Value.ToString();
          if (t is JSSPTask) {
            categoryName = "Job" + ((JSSPTask)t).Job.Index;
            toolTip = categoryName + " - " + toolTip;
          }
          ganttChart.AddCategory(categoryName, Color.FromArgb(random.Next(255), random.Next(255), random.Next(255)));
          ganttChart.AddData("Resource" + r.Index, 
            categoryName, 
            t.StartTime.Value, 
            t.EndTime.Value, 
            toolTip, 
            false);
        }
        resCount++;
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      ganttChart.Enabled = Content != null;
    }
  }
}
